import argparse
import sys

import yoda


def main():

    parser = argparse.ArgumentParser(description="Rename PDF weights in YODA files")

    # Adding optional argument with default value
    parser.add_argument("files", nargs="+", help="YODA files")
    parser.add_argument("-p", "--pdf", type=int, default=27100, help="PDF number")
    parser.add_argument(
        "-n", "--numMembers", type=int, default=65, help="Number of members"
    )
    parser.add_argument("-s", "--start", type=int, default=8, help="Start index")
    parser.add_argument(
        "-e",
        "--end",
        type=int,
        default=-1,
        help="End index, default is numMembers + start - 2",
    )
    parser.add_argument("--prefix", type=str, default="pdf_", help="Output file prefix")

    args = parser.parse_args()

    pdf = args.pdf
    numMembers = args.numMembers
    start = args.start
    end = args.end if args.end > 0 else numMembers + start - 2
    prefix = args.prefix

    weights = {}
    weights[1] = pdf
    for i in range(start, end + 1):
        weights[i] = i + pdf - start + 1

    for f in sys.argv[1:]:
        map = yoda.read(f)
        nmap = {}

        for k, v in map.items():
            n = k
            h = v
            nmap[k] = v
            for w, p in weights.items():
                n = n.replace(f"[W{w}]", f"[PDF{p}]")
                h.setPath(h.path().replace(f"[W{w}]", f"[PDF{p}]"))
            nmap[n] = h

        yoda.write(nmap, args.prefix + f)



#!/usr/bin/env python3
import yoda
from sklearn import metrics

for f in ["../alice_pwhg_cc.yoda", "../alice_pwhg_bb.yoda"]:
    m = yoda.read(f)
    print(f"{f}:")
    for k, s in m.items():
        v = metrics.auc([p.x() for p in s.points()], [p.y() for p in s.points()])
        k = k.replace("ALICE_2020_I1797621", "ALICE_2020_I1797621.*")
        print(f' -c "{k} {v}" ', end="")
    print()



import os
import sys

import pylhe

# Read list of files passed as arguments


if len(sys.argv) < 2:
    print("Usage: %s <lhe files>" % sys.argv[0])
    sys.exit(1)

lhe_files = sys.argv[1:]

# Read the LHE files
for lhe_file in lhe_files:
    events = pylhe.read_lhe_with_attributes(lhe_file)



#!/usr/bin/env python3
import argparse

import yoda
from sklearn import metrics


def main():

    argparser = argparse.ArgumentParser(description="Calculate AUC for YODA files")

    # Adding optional argument with default value
    argparser.add_argument("files", nargs="+", help="YODA files")
    argparser.add_argument(
        "-a",
        "--analysis",
        default="ALICE_2020_I1797621",
        help="analysis name to replace",
    )

    args = argparser.parse_args()

    files = args.files
    analysis = args.analysis

    for f in files:
        m = yoda.read(f)
        print(f"{f}:")
        for k, s in m.items():
            v = metrics.auc([p.x() for p in s.points()], [p.y() for p in s.points()])
            k = k.replace(analysis, analysis + ".*")
            print(f' -c "{k} {v}" ', end="")
        print()



#!/usr/bin/env python3
import argparse


def main():

    parser = argparse.ArgumentParser(description="Generate powheg seeds")

    # Adding optional argument with default value
    parser.add_argument(
        "--n", "--number", type=int, default=2000, help="number of seeds"
    )
    parser.add_argument("-s", "--start", type=int, default=1, help="start value")
    parser.add_argument("-e", "--end", type=int, default=32768, help="end value")
    parser.add_argument(
        "-o", "--output", type=str, default="pwgseeds.dat", help="output file"
    )

    args = parser.parse_args()

    with open(args.output, "w") as f:
        for i in range(args.start, args.end, int(args.end / args.n)):
            print(i, file=f)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import argparse
import os


def main():

    parser = argparse.ArgumentParser(description="Clean up powheg directory")

    # Adding optional argument with default value
    parser.add_argument(
        "--path", default=".", type=str, help="path to powheg directory"
    )

    args = parser.parse_args()

    # remove single file
    for f in [
        "bornequiv",
        "pwg-btlgrid.top",
        "pwggrid.dat",
        "pwgubound.dat",
        "realequivregions-btl",
        "FlavRegList",
        "pwgborngrid.top",
        "pwgcounters.dat",
        "pwghistnorms.top",
        "pwgxgrid.dat",
        "realequivregions-rad",
        "pwgboundviolations.dat",
        "pwgevents.lhe",
        "pwg-stat.dat",
        "pwhg_checklimits",
        "virtequiv",
        "parameters.ol",
    ]:
        p = os.path.join(args.path, f)
        if os.path.exists(p):
            os.remove(p)
    # remove parallel run files
    # TODO


if __name__ == "__main__":
    main()



pdf1 = None
pdf2 = None
# pdf1=27100

pdf = None
pdf = 275200
numMembers = 100
unc_scale = True
unc_pdf = True

s = ""
s += f"<initrwgt>\n"
s += f"<weightgroup name='nominal' combine='None'>\n"
s += f"<weight id='MUR1.0_MUF1.0_PDF{pdf}'>default</weight>\n"
s += f"</weightgroup>\n"
if unc_scale:
    s += f"<weightgroup name='scale_variations' combine='None'>\n"
    s += f"<weight id='MUR2.0_MUF2.0_PDF{pdf}'> renscfact=2d0 facscfact=2d0 </weight>\n"
    s += f"<weight id='MUR0.5_MUF0.5_PDF{pdf}'> renscfact=0.5d0 facscfact=0.5d0 </weight>\n"
    s += f"<weight id='MUR1.0_MUF2.0_PDF{pdf}'> renscfact=1d0 facscfact=2d0 </weight>\n"
    s += f"<weight id='MUR1.0_MUF0.5_PDF{pdf}'> renscfact=1d0 facscfact=0.5d0 </weight>\n"
    s += f"<weight id='MUR2.0_MUF1.0_PDF{pdf}'> renscfact=2d0 facscfact=1d0 </weight>\n"
    s += f"<weight id='MUR0.5_MUF1.0_PDF{pdf}'> renscfact=0.5d0 facscfact=1d0 </weight>\n"
    s += f"</weightgroup>"
if unc_pdf:
    s += f"<weightgroup name='pdf_variations' combine='None'>\n"
    for i in range(1, numMembers):
        if pdf1 is not None and pdf2 is None:
            s += f"<weight id='MUR1.0_MUF1.0_PDF{pdf+i}'> lhapdf1={pdf1} lhapdf2={pdf+i}</weight>\n"
        elif pdf2 is not None and pdf1 is None:
            s += f"<weight id='MUR1.0_MUF1.0_PDF{pdf+i}'> lhapdf1={pdf+i} lhapdf2={pdf2}</weight>\n"
        elif pdf1 is None and pdf2 is None:
            s += f"<weight id='MUR1.0_MUF1.0_PDF{pdf+i}'> lhapdf={pdf+i}</weight>\n"
        else:
            print("FAILED")
            exit(1)

    s += f"</weightgroup>\n"
s += f"</initrwgt>\n"

print(s)



#!/usr/bin/env python3
import argparse


def pdfidtonummembers(pdfid):
    import lhapdf

    print("No number of members specified => trying LHAPDF")
    pdf = lhapdf.mkPDF(pdfid)
    pdfsetname = pdf.set().name
    pdfset = lhapdf.mkPDFs(pdfsetname)
    return len(pdfset)


def pdftoid(pdfname):
    if pdfname is None:
        return None
    elif pdfname.isdigit():
        return int(pdfname)
    else:
        print(f"PDF name {pdfname} is not a number => trying LHAPDF")
        import lhapdf

        pdfset = lhapdf.mkPDF(f"{pdfname}")
        return pdfset.lhapdfID


def main():
    parser = argparse.ArgumentParser(description="Generate initrwgt block for Powheg")

    # Adding optional argument with default value
    parser.add_argument("pdf", type=str, help="PDF number 1 and/or 2")
    parser.add_argument("-p1", "--pdf1", type=str, default=None, help="PDF number 1")
    parser.add_argument("-p2", "--pdf2", type=str, default=None, help="PDF number 2")
    parser.add_argument(
        "-n",
        "--nummembers",
        type=int,
        default=None,
        help="Number of members in PDF set",
    )
    parser.add_argument(
        "-s", "--scalevar", type=bool, default=True, help="Scale variations"
    )
    parser.add_argument(
        "-m", "--pdfvar", type=bool, default=True, help="PDF variations"
    )
    parser.add_argument("-o", "--output", type=str, default=None, help="Output file")

    args = parser.parse_args()

    pdf1 = pdftoid(args.pdf1)
    pdf2 = pdftoid(args.pdf2)

    if pdf1 is not None and pdf2 is not None:
        print("Only one of pdf1 and pdf2 can be set (other pdf is pdf)")
        exit(1)

    pdf = pdftoid(args.pdf)

    numMembers = args.nummembers
    if numMembers is None:
        numMembers = pdfidtonummembers(pdf)

    unc_scale = args.scalevar
    unc_pdf = args.pdfvar

    s = ""
    s += f"<initrwgt>\n"
    s += f"<weightgroup name='nominal' combine='None'>\n"
    s += f"<weight id='MUR1.0_MUF1.0_PDF{pdf}'>default</weight>\n"
    s += f"</weightgroup>\n"
    if unc_scale:
        s += f"<weightgroup name='scale_variations' combine='None'>\n"
        s += f"<weight id='MUR2.0_MUF2.0_PDF{pdf}'> renscfact=2d0 facscfact=2d0 </weight>\n"
        s += f"<weight id='MUR0.5_MUF0.5_PDF{pdf}'> renscfact=0.5d0 facscfact=0.5d0 </weight>\n"
        s += f"<weight id='MUR1.0_MUF2.0_PDF{pdf}'> renscfact=1d0 facscfact=2d0 </weight>\n"
        s += f"<weight id='MUR1.0_MUF0.5_PDF{pdf}'> renscfact=1d0 facscfact=0.5d0 </weight>\n"
        s += f"<weight id='MUR2.0_MUF1.0_PDF{pdf}'> renscfact=2d0 facscfact=1d0 </weight>\n"
        s += f"<weight id='MUR0.5_MUF1.0_PDF{pdf}'> renscfact=0.5d0 facscfact=1d0 </weight>\n"
        s += f"</weightgroup>"
    if unc_pdf:
        s += f"<weightgroup name='pdf_variations' combine='None'>\n"
        for i in range(1, numMembers):
            if pdf1 is not None and pdf2 is None:
                s += f"<weight id='MUR1.0_MUF1.0_PDF{pdf+i}'> lhapdf1={pdf1} lhapdf2={pdf+i}</weight>\n"
            elif pdf2 is not None and pdf1 is None:
                s += f"<weight id='MUR1.0_MUF1.0_PDF{pdf+i}'> lhapdf1={pdf+i} lhapdf2={pdf2}</weight>\n"
            elif pdf1 is None and pdf2 is None:
                s += f"<weight id='MUR1.0_MUF1.0_PDF{pdf+i}'> lhapdf={pdf+i}</weight>\n"
            else:
                print("FAILED")
                exit(1)

        s += f"</weightgroup>\n"
    s += f"</initrwgt>\n"

    if args.output is not None:
        with open(args.output, "w") as f:
            f.write(s)
    else:
        print()
        print(s)


if __name__ == "__main__":
    main()



#!/usr/bin/python

import sys

import numpy as np
import pylhe

abs_max = [(0, 0, "")]

rel_max = [(0, 0, "")]

for arg in sys.argv[1:]:
    for i, event in enumerate(pylhe.read_lhe_with_attributes(arg)):
        # print(event.particles)
        # print(event.weights)
        warr = np.array(list(event.weights.values()))
        warr = warr[7:]
        # print(warr)
        max_difference = np.max(np.abs(np.subtract.outer(warr, warr)))
        # relative difference
        rel_difference = max_difference / np.average(warr)

        if max_difference > abs_max[-1][0]:
            abs_max.append((max_difference, i, arg))
            abs_max.sort(reverse=True, key=lambda x: x[0])
            abs_max = abs_max[:10]
        if rel_difference > rel_max[-1][0]:
            rel_max.append((rel_difference, i, arg))
            rel_max.sort(reverse=True, key=lambda x: x[0])
            rel_max = rel_max[:10]
        # print(i,max_difference)
    # print(pylhe.read_lhe_init(arg))
print(sys.argv[1:])
print("abs_max")
print(abs_max)
print("rel_max")
print(rel_max)
